ProgressLogger

Struct ProgressLogger 

pub struct ProgressLogger { /* private fields */ }
An implementation of ProgressLog with output generated using the log crate at a configurable level (default: info).

Instances can be created by using fluent setters, or by using the progress_logger macro.

You can clone a logger to create a new one with the same setup but with all the counters reset and the expected number of updates cleared. This behavior is useful when you want to configure a logger and then use its configuration for other loggers.

Examples

A typical call sequence to a progress logger is as follows:

env_logger::builder().filter_level(log::LevelFilter::Info).try_init()?;

let mut pl = ProgressLogger::default();
pl.item_name("pumpkin");
pl.start("Smashing pumpkins...");
for _ in 0..100 {
   // do something on each pumpkin
   pl.update();
}
pl.done();

The progress_logger macro will create the progress logger for you and set its log_target to std::module_path!(), which is usually what you want. You can also call any setter with a key-value syntax:

env_logger::builder().filter_level(log::LevelFilter::Info).try_init()?;

let mut pl = progress_logger![item_name="pumpkin"];
pl.start("Smashing pumpkins...");
for _ in 0..100 {
   // do something on each pumpkin
   pl.update();
}
pl.done();

A progress logger can also be used as a handy timer:

env_logger::builder().filter_level(log::LevelFilter::Info).try_init()?;

let mut pl = progress_logger![item_name="pumpkin"];
pl.start("Smashing pumpkins...");
for _ in 0..100 {
   // do something on each pumpkin
}
pl.done_with_count(100);

This progress logger will display information about memory usage:

env_logger::builder().filter_level(log::LevelFilter::Info).try_init()?;

let mut pl = progress_logger![display_memory=true];

Implementations

Source

impl ProgressLogger

pub const LIGHT_UPDATE_MASK: usize

Calls to light_update will cause a call to Instant::now only if the current count is a multiple of this mask plus one.